/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSStyleDeclaration.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomCSSStyleDeclaration.h"
#include "gdom/GdomCSSValue.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/PrivateGdomCSSStyleDeclaration.h"
#include "gdom/PrivateGdomCSSValue.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateIntGdomCSSStyleDeclaration.h"
#include "gdom/PrivateIntGdomCSSValue.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::CSSStyleDeclaration* obj)
{
    return WebCore::cacheDOMObject<WebCore::CSSStyleDeclaration, GDOMCSSStyleDeclaration>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMCSSStyleDeclaration::GDOMCSSStyleDeclaration(PassRefPtr<WebCore::CSSStyleDeclaration> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMCSSStyleDeclaration::~GDOMCSSStyleDeclaration()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

gchar *
gdom_css_style_declaration_get_property_value (GdomCSSStyleDeclaration *thiz, gchar * property_name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CSSStyleDeclaration * item = (gdom_css_style_declaration_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (property_name, 0);
    WebCore::String _g_property_name = WebCore::String::fromUTF8(property_name);
    gchar * res = _G_GET_UTF8(item->getPropertyValue(_g_property_name));
    return res;

}

GdomCSSValue *
gdom_css_style_declaration_get_property_css_value (GdomCSSStyleDeclaration *thiz, gchar * property_name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CSSStyleDeclaration * item = (gdom_css_style_declaration_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (property_name, 0);
    WebCore::String _g_property_name = WebCore::String::fromUTF8(property_name);
    PassRefPtr<WebCore::CSSValue> g_res = WTF::getPtr(item->getPropertyCSSValue(_g_property_name));
    GdomCSSValue * res = static_cast<GdomCSSValue *>(WebCore::toGDOM(g_res.get()));
    return res;

}

gchar *
gdom_css_style_declaration_remove_property (GdomCSSStyleDeclaration *thiz, gchar * property_name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CSSStyleDeclaration * item = (gdom_css_style_declaration_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (property_name, 0);
    WebCore::String _g_property_name = WebCore::String::fromUTF8(property_name);
    WebCore::ExceptionCode ec = 0;
    gchar * res = _G_GET_UTF8(item->removeProperty(_g_property_name, ec));
    return res;

}

gchar *
gdom_css_style_declaration_get_property_priority (GdomCSSStyleDeclaration *thiz, gchar * property_name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CSSStyleDeclaration * item = (gdom_css_style_declaration_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (property_name, 0);
    WebCore::String _g_property_name = WebCore::String::fromUTF8(property_name);
    gchar * res = _G_GET_UTF8(item->getPropertyPriority(_g_property_name));
    return res;

}

void
gdom_css_style_declaration_set_css_property (GdomCSSStyleDeclaration *thiz, gchar * property_name, gchar * value, gchar * priority)
{
    g_return_if_fail (thiz);
    WebCore::CSSStyleDeclaration * item = (gdom_css_style_declaration_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (property_name);
    g_return_if_fail (value);
    g_return_if_fail (priority);
    WebCore::String _g_property_name = WebCore::String::fromUTF8(property_name);
    WebCore::String _g_value = WebCore::String::fromUTF8(value);
    WebCore::String _g_priority = WebCore::String::fromUTF8(priority);
    WebCore::ExceptionCode ec = 0;
    item->setProperty(_g_property_name, _g_value, _g_priority, ec);

}

gchar *
gdom_css_style_declaration_item (GdomCSSStyleDeclaration *thiz, gulong index)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CSSStyleDeclaration * item = (gdom_css_style_declaration_get_private(thiz));
    g_return_val_if_fail (item, 0);
    gchar * res = _G_GET_UTF8(item->item(index));
    return res;

}

gchar *
gdom_css_style_declaration_get_property_shorthand (GdomCSSStyleDeclaration *thiz, gchar * property_name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CSSStyleDeclaration * item = (gdom_css_style_declaration_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (property_name, 0);
    WebCore::String _g_property_name = WebCore::String::fromUTF8(property_name);
    gchar * res = _G_GET_UTF8(item->getPropertyShorthand(_g_property_name));
    return res;

}

gboolean
gdom_css_style_declaration_is_property_implicit (GdomCSSStyleDeclaration *thiz, gchar * property_name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CSSStyleDeclaration * item = (gdom_css_style_declaration_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (property_name, 0);
    WebCore::String _g_property_name = WebCore::String::fromUTF8(property_name);
    gboolean res = item->isPropertyImplicit(_g_property_name);
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "CSSStyleDeclaration.h"
#include <gdom/GdomCSSStyleDeclaration.h>
#include <gdom/PrivateGdomCSSStyleDeclaration.h>
#include <gdom/PrivateIntGdomCSSStyleDeclaration.h>
#include <gdom/GdomCSSStyleDeclarationprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomCSSStyleDeclaration, gdom_css_style_declaration, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomCSSStyleDeclarationPrivate {
    class WebCore::CSSStyleDeclaration *item;
};

#define GDOM_CSS_STYLE_DECLARATION_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_CSS_STYLE_DECLARATION, GdomCSSStyleDeclarationPrivate))
*/
G_END_DECLS
WebCore::CSSStyleDeclaration *
gdom_css_style_declaration_get_private(GdomCSSStyleDeclaration *request)
{
    GDOMCSSStyleDeclaration *res;
    res = static_cast<GDOMCSSStyleDeclaration*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::CSSStyleDeclaration*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_CSS_TEXT,
    PROP_LENGTH,
    PROP_PARENT_RULE,
};


static void gdom_css_style_declaration_finalize(GObject* object)
{
    /*GdomCSSStyleDeclaration* request = GDOM_CSS_STYLE_DECLARATION(object);*/
    /*GdomCSSStyleDeclarationPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_css_style_declaration_parent_class)->finalize(object);
}

static void gdom_css_style_declaration__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomCSSStyleDeclaration *self = GDOM_CSS_STYLE_DECLARATION(object);
    WebCore::CSSStyleDeclaration * gself = (gdom_css_style_declaration_get_private(self));
    switch (prop_id) {
     case PROP_CSS_TEXT:
    {
         WebCore::ExceptionCode ec = 0;
         gself->setCssText(WebCore::String::fromUTF8(g_value_get_string (value)) , ec );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_css_style_declaration__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomCSSStyleDeclaration *self = GDOM_CSS_STYLE_DECLARATION(object);
    WebCore::CSSStyleDeclaration * gself = (gdom_css_style_declaration_get_private(self));
    switch (prop_id) {
    case PROP_CSS_TEXT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->cssText()));
         break;
    }
    case PROP_LENGTH:
    {
         g_value_set_ulong (value, gself->length());
         break;
    }
    case PROP_PARENT_RULE:
    {
         PassRefPtr<WebCore::CSSRule> ptr = gself->parentRule();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_css_style_declaration_class_init(GdomCSSStyleDeclarationClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_css_style_declaration_finalize;
    g_klass->set_property = gdom_css_style_declaration__set_property_fn;
    g_klass->get_property = gdom_css_style_declaration__get_property_fn;

    g_object_class_install_property (g_klass, PROP_CSS_TEXT,
            g_param_spec_string ("css_text", /* name */
                "css_style_declaration_css_text", /* short description */
                "read-write  gchar * CSSStyleDeclaration.css_text", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_LENGTH,
            g_param_spec_ulong ("length", /* name */
                "css_style_declaration_length", /* short description */
                "read-only  gulong CSSStyleDeclaration.length", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PARENT_RULE,
            g_param_spec_object ("parent_rule", /* name */
                "css_style_declaration_parent_rule", /* short description */
                "read-only  GdomCSSRule * CSSStyleDeclaration.parent_rule", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_CSS_RULE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomCSSStyleDeclarationPrivate));*/
}

static void gdom_css_style_declaration_init(GdomCSSStyleDeclaration* request)
{
    /*GdomCSSStyleDeclarationPrivate* priv = GDOM_CSS_STYLE_DECLARATION_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

